from _typeshed import Incomplete

LOGGER: Incomplete

def name_or_value(value): ...
def sanitize_prefix(function): ...
def check_for_prefix_and_key(function): ...

class CallbackManager:
    CALLS: str
    ARGUMENTS: str
    DUPLICATE_WARNING: str
    CALLBACK: str
    ONE_SHOT: str
    ONLY_CALLER: str
    def __init__(self) -> None: ...
    def add(
        self,
        prefix,
        key,
        callback,
        one_shot: bool = True,
        only_caller: Incomplete | None = None,
        arguments: Incomplete | None = None,
    ): ...
    def clear(self) -> None: ...
    def cleanup(self, prefix): ...
    def pending(self, prefix, key): ...
    def process(self, prefix, key, caller, *args, **keywords): ...
    def remove(self, prefix, key, callback_value: Incomplete | None = None, arguments: Incomplete | None = None): ...
    def remove_all(self, prefix, key) -> None: ...



import abc
from _typeshed import Incomplete
from collections.abc import Callable
from typing_extensions import Self

from ..adapters.utils import nbio_interface
from ..connection import Connection

LOGGER: Incomplete

class BaseConnection(Connection, metaclass=abc.ABCMeta):
    def __init__(
        self,
        parameters,
        on_open_callback: Callable[[Self], object] | None,
        on_open_error_callback: Callable[[Self, BaseException], object] | None,
        on_close_callback: Callable[[Self, BaseException], object] | None,
        nbio,
        internal_connection_workflow: bool,
    ) -> None: ...
    @classmethod
    @abc.abstractmethod
    def create_connection(
        cls, connection_configs, on_done, custom_ioloop: Incomplete | None = None, workflow: Incomplete | None = None
    ): ...
    @property
    def ioloop(self): ...

class _StreamingProtocolShim(nbio_interface.AbstractStreamProtocol):
    connection_made: Incomplete
    connection_lost: Incomplete
    eof_received: Incomplete
    data_received: Incomplete
    conn: Incomplete
    def __init__(self, conn) -> None: ...
    def __getattr__(self, attr: str): ...



from collections.abc import Mapping
from datetime import datetime
from decimal import Decimal
from typing_extensions import TypeAlias

_Value: TypeAlias = str | bytes | bool | int | Decimal | datetime | _ArgumentMapping | list[_Value] | None
_ArgumentMapping: TypeAlias = Mapping[str, _Value]

def encode_short_string(pieces: list[bytes], value: str | bytes) -> int: ...
def decode_short_string(encoded: bytes, offset: int) -> tuple[str, int]: ...
def encode_table(pieces: list[bytes], table: _ArgumentMapping) -> int: ...
def encode_value(pieces: list[bytes], value: _Value) -> int: ...
def decode_table(encoded: bytes, offset: int) -> tuple[dict[str, _Value], int]: ...
def decode_value(encoded: bytes, offset: int) -> tuple[_Value, int]: ...



# twisted is optional and self-contained in this module.
# We don't want to force it as a dependency but that means we also can't test it with type-checkers given the current setup.

from _typeshed import Incomplete
from typing import NamedTuple, TypeVar

import pika.connection
from pika.adapters.utils import nbio_interface
from twisted.internet.base import DelayedCall  # type: ignore[import]  # pyright: ignore[reportMissingImports]
from twisted.internet.defer import Deferred, DeferredQueue  # type: ignore[import]  # pyright: ignore[reportMissingImports]
from twisted.internet.interfaces import ITransport  # type: ignore[import]  # pyright: ignore[reportMissingImports]
from twisted.internet.protocol import Protocol  # type: ignore[import]  # pyright: ignore[reportMissingImports]
from twisted.python.failure import Failure  # type: ignore[import]  # pyright: ignore[reportMissingImports]

_T = TypeVar("_T")

LOGGER: Incomplete

class ClosableDeferredQueue(DeferredQueue[_T]):  # pyright: ignore[reportUntypedBaseClass]
    closed: Failure | BaseException | None
    def __init__(self, size: Incomplete | None = ..., backlog: Incomplete | None = ...) -> None: ...
    # Returns a Deferred with an error if fails. None if success
    def put(self, obj: _T) -> Deferred[Failure | BaseException] | None: ...  # type: ignore[override]  # pyright: ignore[reportInvalidTypeVarUse]
    def get(self) -> Deferred[Failure | BaseException | _T]: ...
    pending: Incomplete
    def close(self, reason: BaseException | None) -> None: ...

class ReceivedMessage(NamedTuple):
    channel: Incomplete
    method: Incomplete
    properties: Incomplete
    body: Incomplete

class TwistedChannel:
    on_closed: Deferred[Incomplete | Failure | BaseException | None]
    def __init__(self, channel) -> None: ...
    @property
    def channel_number(self): ...
    @property
    def connection(self): ...
    @property
    def is_closed(self): ...
    @property
    def is_closing(self): ...
    @property
    def is_open(self): ...
    @property
    def flow_active(self): ...
    @property
    def consumer_tags(self): ...
    def callback_deferred(self, deferred, replies) -> None: ...
    def add_on_return_callback(self, callback): ...
    def basic_ack(self, delivery_tag: int = ..., multiple: bool = ...): ...
    def basic_cancel(self, consumer_tag: str = ...) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def basic_consume(
        self,
        queue,
        auto_ack: bool = ...,
        exclusive: bool = ...,
        consumer_tag: Incomplete | None = ...,
        arguments: Incomplete | None = ...,
    ) -> Deferred[Incomplete | Failure | BaseException]: ...
    def basic_get(self, queue, auto_ack: bool = ...) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def basic_nack(self, delivery_tag: Incomplete | None = ..., multiple: bool = ..., requeue: bool = ...): ...
    def basic_publish(
        self, exchange, routing_key, body, properties: Incomplete | None = ..., mandatory: bool = ...
    ) -> Deferred[Incomplete | Failure | BaseException]: ...
    def basic_qos(
        self, prefetch_size: int = ..., prefetch_count: int = ..., global_qos: bool = ...
    ) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def basic_reject(self, delivery_tag, requeue: bool = ...): ...
    def basic_recover(self, requeue: bool = ...) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def close(self, reply_code: int = ..., reply_text: str = ...): ...
    def confirm_delivery(self) -> Deferred[Incomplete | None]: ...
    def exchange_bind(
        self, destination, source, routing_key: str = ..., arguments: Incomplete | None = ...
    ) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def exchange_declare(
        self,
        exchange,
        exchange_type=...,
        passive: bool = ...,
        durable: bool = ...,
        auto_delete: bool = ...,
        internal: bool = ...,
        arguments: Incomplete | None = ...,
    ) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def exchange_delete(
        self, exchange: Incomplete | None = ..., if_unused: bool = ...
    ) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def exchange_unbind(
        self,
        destination: Incomplete | None = ...,
        source: Incomplete | None = ...,
        routing_key: str = ...,
        arguments: Incomplete | None = ...,
    ) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def flow(self, active) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def open(self): ...
    def queue_bind(
        self, queue, exchange, routing_key: Incomplete | None = ..., arguments: Incomplete | None = ...
    ) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def queue_declare(
        self,
        queue,
        passive: bool = ...,
        durable: bool = ...,
        exclusive: bool = ...,
        auto_delete: bool = ...,
        arguments: Incomplete | None = ...,
    ) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def queue_delete(
        self, queue, if_unused: bool = ..., if_empty: bool = ...
    ) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def queue_purge(self, queue) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def queue_unbind(
        self, queue, exchange: Incomplete | None = ..., routing_key: Incomplete | None = ..., arguments: Incomplete | None = ...
    ) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def tx_commit(self) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def tx_rollback(self) -> Deferred[Incomplete | Failure | BaseException | None]: ...
    def tx_select(self) -> Deferred[Incomplete | Failure | BaseException | None]: ...

class _TwistedConnectionAdapter(pika.connection.Connection):
    def __init__(self, parameters, on_open_callback, on_open_error_callback, on_close_callback, custom_reactor) -> None: ...
    def connection_made(self, transport: ITransport) -> None: ...
    def connection_lost(self, error: Exception) -> None: ...
    def data_received(self, data) -> None: ...

class TwistedProtocolConnection(Protocol):  # pyright: ignore[reportUntypedBaseClass]
    ready: Deferred[None] | None
    closed: Deferred[None] | Failure | BaseException | None
    def __init__(self, parameters: Incomplete | None = ..., custom_reactor: Incomplete | None = ...) -> None: ...
    def channel(self, channel_number: Incomplete | None = ...): ...
    @property
    def is_open(self): ...
    @property
    def is_closed(self): ...
    def close(self, reply_code: int = ..., reply_text: str = ...) -> Deferred[None] | Failure | BaseException | None: ...
    def dataReceived(self, data) -> None: ...
    def connectionLost(self, reason: Failure | BaseException = ...) -> None: ...
    def makeConnection(self, transport: ITransport) -> None: ...
    def connectionReady(self): ...

class _TimerHandle(nbio_interface.AbstractTimerReference):
    def __init__(self, handle: DelayedCall) -> None: ...
    def cancel(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable
from logging import Logger
from typing import Any
from typing_extensions import Final, Self

from .callback import CallbackManager
from .connection import Connection
from .data import _ArgumentMapping
from .exchange_type import ExchangeType
from .frame import Body, Header, Method
from .spec import Basic, BasicProperties, Confirm, Exchange, Queue, Tx

LOGGER: Logger
MAX_CHANNELS: Final[int]

class Channel:
    CLOSED: Final = 0
    OPENING: Final = 1
    OPEN: Final = 2
    CLOSING: Final = 3

    channel_number: int
    callbacks: CallbackManager
    connection: Connection
    flow_active: bool

    def __init__(self, connection: Connection, channel_number: int, on_open_callback: Callable[[Self], object]) -> None: ...
    def __int__(self) -> int: ...
    def add_callback(self, callback, replies, one_shot: bool = True) -> None: ...
    def add_on_cancel_callback(self, callback) -> None: ...
    def add_on_close_callback(self, callback) -> None: ...
    def add_on_flow_callback(self, callback) -> None: ...
    def add_on_return_callback(self, callback) -> None: ...
    def basic_ack(self, delivery_tag: int = 0, multiple: bool = False) -> None: ...
    def basic_cancel(
        self, consumer_tag: str = "", callback: Callable[[Method[Basic.CancelOk]], object] | None = None
    ) -> None: ...
    def basic_consume(
        self,
        queue: str,
        on_message_callback: Callable[[Channel, Basic.Deliver, BasicProperties, bytes], object],
        auto_ack: bool = False,
        exclusive: bool = False,
        consumer_tag: str | None = None,
        arguments: _ArgumentMapping | None = None,
        callback: Callable[[Method[Basic.ConsumeOk]], object] | None = None,
    ) -> str: ...
    def basic_get(
        self, queue: str, callback: Callable[[Channel, Basic.GetOk, BasicProperties, bytes], object], auto_ack: bool = False
    ) -> None: ...
    def basic_nack(self, delivery_tag: int = 0, multiple: bool = False, requeue: bool = True) -> None: ...
    def basic_publish(
        self,
        exchange: str,
        routing_key: str,
        body: str | bytes,
        properties: BasicProperties | None = None,
        mandatory: bool = False,
    ) -> None: ...
    def basic_qos(
        self,
        prefetch_size: int = 0,
        prefetch_count: int = 0,
        global_qos: bool = False,
        callback: Callable[[Method[Basic.QosOk]], object] | None = None,
    ) -> None: ...
    def basic_reject(self, delivery_tag: int = 0, requeue: bool = True) -> None: ...
    def basic_recover(
        self, requeue: bool = False, callback: Callable[[Method[Basic.RecoverOk]], object] | None = None
    ) -> None: ...
    def close(self, reply_code: int = 0, reply_text: str = "Normal shutdown") -> None: ...
    def confirm_delivery(
        self,
        ack_nack_callback: Callable[[Method[Basic.Ack | Basic.Nack]], object],
        callback: Callable[[Method[Confirm.SelectOk]], object] | None = None,
    ) -> None: ...
    @property
    def consumer_tags(self) -> list[str]: ...
    def exchange_bind(
        self,
        destination: str,
        source: str,
        routing_key: str = "",
        arguments: _ArgumentMapping | None = None,
        callback: Callable[[Method[Exchange.BindOk]], object] | None = None,
    ) -> None: ...
    def exchange_declare(
        self,
        exchange: str,
        exchange_type: ExchangeType | str = ...,
        passive: bool = False,
        durable: bool = False,
        auto_delete: bool = False,
        internal: bool = False,
        arguments: _ArgumentMapping | None = None,
        callback: Callable[[Method[Exchange.DeclareOk]], object] | None = None,
    ) -> None: ...
    def exchange_delete(
        self,
        exchange: str | None = None,
        if_unused: bool = False,
        callback: Callable[[Method[Exchange.DeleteOk]], object] | None = None,
    ) -> None: ...
    def exchange_unbind(
        self,
        destination: str | None = None,
        source: str | None = None,
        routing_key: str = "",
        arguments: _ArgumentMapping | None = None,
        callback: Callable[[Method[Exchange.UnbindOk]], object] | None = None,
    ) -> None: ...
    def flow(self, active: bool, callback: Callable[[bool], object] | None = None) -> None: ...
    @property
    def is_closed(self) -> bool: ...
    @property
    def is_closing(self) -> bool: ...
    @property
    def is_open(self) -> bool: ...
    @property
    def is_opening(self) -> bool: ...
    def open(self) -> None: ...
    def queue_bind(
        self,
        queue: str,
        exchange: str,
        routing_key: str | None = None,
        arguments: _ArgumentMapping | None = None,
        callback: Callable[[Method[Queue.BindOk]], object] | None = None,
    ) -> None: ...
    def queue_declare(
        self,
        queue: str,
        passive: bool = False,
        durable: bool = False,
        exclusive: bool = False,
        auto_delete: bool = False,
        arguments: _ArgumentMapping | None = None,
        callback: Callable[[Method[Queue.DeclareOk]], object] | None = None,
    ) -> None: ...
    def queue_delete(
        self,
        queue: str,
        if_unused: bool = False,
        if_empty: bool = False,
        callback: Callable[[Method[Queue.DeleteOk]], object] | None = None,
    ) -> None: ...
    def queue_purge(self, queue: str, callback: Callable[[Method[Queue.PurgeOk]], object] | None = None) -> None: ...
    def queue_unbind(
        self,
        queue: str,
        exchange: str | None = None,
        routing_key: str | None = None,
        arguments: _ArgumentMapping | None = None,
        callback: Callable[[Method[Queue.UnbindOk]], object] | None = None,
    ): ...
    def tx_commit(self, callback: Callable[[Method[Tx.CommitOk]], object] | None = None) -> None: ...
    def tx_rollback(self, callback: Callable[[Method[Tx.RollbackOk]], object] | None = None) -> None: ...
    def tx_select(self, callback: Callable[[Method[Tx.SelectOk]], object] | None = None) -> None: ...

class ContentFrameAssembler:
    def __init__(self) -> None: ...
    def process(self, frame_value: Method[Any] | Header | Body) -> tuple[Incomplete, Incomplete, bytes] | None: ...



from _typeshed import Incomplete

LOGGER: Incomplete

class HeartbeatChecker:
    def __init__(self, connection, timeout) -> None: ...
    @property
    def bytes_received_on_connection(self): ...
    @property
    def connection_is_idle(self): ...
    def received(self) -> None: ...
    def stop(self) -> None: ...



from _typeshed import Incomplete, Unused
from collections.abc import Generator, Sequence
from types import TracebackType
from typing import NamedTuple
from typing_extensions import Self

from ..connection import Parameters
from ..data import _ArgumentMapping
from ..exchange_type import ExchangeType
from ..spec import BasicProperties

LOGGER: Incomplete

class _CallbackResult:
    def __init__(self, value_class: Incomplete | None = None) -> None: ...
    def reset(self) -> None: ...
    def __bool__(self) -> bool: ...
    __nonzero__: Incomplete
    def __enter__(self): ...
    def __exit__(self, *args: Unused, **kwargs: Unused) -> None: ...
    def is_ready(self): ...
    @property
    def ready(self): ...
    def signal_once(self, *_args, **_kwargs) -> None: ...
    def set_value_once(self, *args, **kwargs) -> None: ...
    def append_element(self, *args, **kwargs) -> None: ...
    @property
    def value(self): ...
    @property
    def elements(self): ...

class _IoloopTimerContext:
    def __init__(self, duration, connection) -> None: ...
    def __enter__(self): ...
    def __exit__(self, *_args: Unused, **_kwargs: Unused) -> None: ...
    def is_ready(self): ...

class _TimerEvt:
    timer_id: Incomplete
    def __init__(self, callback) -> None: ...
    def dispatch(self) -> None: ...

class _ConnectionBlockedUnblockedEvtBase:
    def __init__(self, callback, method_frame) -> None: ...
    def dispatch(self) -> None: ...

class _ConnectionBlockedEvt(_ConnectionBlockedUnblockedEvtBase): ...
class _ConnectionUnblockedEvt(_ConnectionBlockedUnblockedEvtBase): ...

class BlockingConnection:
    class _OnClosedArgs(NamedTuple):
        connection: Incomplete
        error: Incomplete

    class _OnChannelOpenedArgs(NamedTuple):
        channel: Incomplete
    def __init__(
        self, parameters: Parameters | Sequence[Parameters] | None = None, _impl_class: Incomplete | None = None
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def add_on_connection_blocked_callback(self, callback) -> None: ...
    def add_on_connection_unblocked_callback(self, callback) -> None: ...
    def call_later(self, delay, callback): ...
    def add_callback_threadsafe(self, callback) -> None: ...
    def remove_timeout(self, timeout_id) -> None: ...
    def update_secret(self, new_secret, reason) -> None: ...
    def close(self, reply_code: int = 200, reply_text: str = "Normal shutdown") -> None: ...
    def process_data_events(self, time_limit: int = 0): ...
    def sleep(self, duration: float) -> None: ...
    def channel(self, channel_number: int | None = None) -> BlockingChannel: ...
    @property
    def is_closed(self) -> bool: ...
    @property
    def is_open(self) -> bool: ...
    @property
    def basic_nack_supported(self) -> bool: ...
    @property
    def consumer_cancel_notify_supported(self) -> bool: ...
    @property
    def exchange_exchange_bindings_supported(self) -> bool: ...
    @property
    def publisher_confirms_supported(self) -> bool: ...
    basic_nack = basic_nack_supported
    consumer_cancel_notify = consumer_cancel_notify_supported
    exchange_exchange_bindings = exchange_exchange_bindings_supported
    publisher_confirms = publisher_confirms_supported

class _ChannelPendingEvt: ...

class _ConsumerDeliveryEvt(_ChannelPendingEvt):
    method: Incomplete
    properties: Incomplete
    body: Incomplete
    def __init__(self, method, properties, body) -> None: ...

class _ConsumerCancellationEvt(_ChannelPendingEvt):
    method_frame: Incomplete
    def __init__(self, method_frame) -> None: ...
    @property
    def method(self): ...

class _ReturnedMessageEvt(_ChannelPendingEvt):
    callback: Incomplete
    channel: Incomplete
    method: Incomplete
    properties: Incomplete
    body: Incomplete
    def __init__(self, callback, channel, method, properties, body) -> None: ...
    def dispatch(self) -> None: ...

class ReturnedMessage:
    method: Incomplete
    properties: Incomplete
    body: Incomplete
    def __init__(self, method, properties, body) -> None: ...

class _ConsumerInfo:
    SETTING_UP: int
    ACTIVE: int
    TEARING_DOWN: int
    CANCELLED_BY_BROKER: int
    consumer_tag: Incomplete
    auto_ack: Incomplete
    on_message_callback: Incomplete
    alternate_event_sink: Incomplete
    state: Incomplete
    def __init__(
        self,
        consumer_tag,
        auto_ack,
        on_message_callback: Incomplete | None = None,
        alternate_event_sink: Incomplete | None = None,
    ) -> None: ...
    @property
    def setting_up(self): ...
    @property
    def active(self): ...
    @property
    def tearing_down(self): ...
    @property
    def cancelled_by_broker(self): ...

class _QueueConsumerGeneratorInfo:
    params: Incomplete
    consumer_tag: Incomplete
    pending_events: Incomplete
    def __init__(self, params, consumer_tag) -> None: ...

class BlockingChannel:
    class _RxMessageArgs(NamedTuple):
        channel: Incomplete
        method: Incomplete
        properties: Incomplete
        body: Incomplete

    class _MethodFrameCallbackResultArgs(NamedTuple):
        method_frame: Incomplete

    class _OnMessageConfirmationReportArgs(NamedTuple):
        method_frame: Incomplete

    class _FlowOkCallbackResultArgs(NamedTuple):
        active: Incomplete
    def __init__(self, channel_impl, connection) -> None: ...
    def __int__(self) -> int: ...
    def __enter__(self): ...
    def __exit__(
        self, exc_type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    @property
    def channel_number(self): ...
    @property
    def connection(self): ...
    @property
    def is_closed(self): ...
    @property
    def is_open(self): ...
    @property
    def consumer_tags(self): ...
    def close(self, reply_code: int = 0, reply_text: str = "Normal shutdown"): ...
    def flow(self, active): ...
    def add_on_cancel_callback(self, callback) -> None: ...
    def add_on_return_callback(self, callback): ...
    def basic_consume(
        self,
        queue,
        on_message_callback,
        auto_ack: bool = False,
        exclusive: bool = False,
        consumer_tag: Incomplete | None = None,
        arguments: Incomplete | None = None,
    ): ...
    def basic_cancel(self, consumer_tag): ...
    def start_consuming(self) -> None: ...
    def stop_consuming(self, consumer_tag: Incomplete | None = None) -> None: ...
    def consume(
        self,
        queue,
        auto_ack: bool = False,
        exclusive: bool = False,
        arguments: Incomplete | None = None,
        inactivity_timeout: Incomplete | None = None,
    ) -> Generator[Incomplete, None, None]: ...
    def get_waiting_message_count(self): ...
    def cancel(self): ...
    def basic_ack(self, delivery_tag: int = 0, multiple: bool = False) -> None: ...
    def basic_nack(self, delivery_tag: int = 0, multiple: bool = False, requeue: bool = True) -> None: ...
    def basic_get(self, queue, auto_ack: bool = False): ...
    def basic_publish(
        self,
        exchange: str,
        routing_key: str,
        body: str | bytes,
        properties: BasicProperties | None = None,
        mandatory: bool = False,
    ) -> None: ...
    def basic_qos(self, prefetch_size: int = 0, prefetch_count: int = 0, global_qos: bool = False) -> None: ...
    def basic_recover(self, requeue: bool = False) -> None: ...
    def basic_reject(self, delivery_tag: int = 0, requeue: bool = True) -> None: ...
    def confirm_delivery(self) -> None: ...
    def exchange_declare(
        self,
        exchange: str,
        exchange_type: ExchangeType | str = ...,
        passive: bool = False,
        durable: bool = False,
        auto_delete: bool = False,
        internal: bool = False,
        arguments: _ArgumentMapping | None = None,
    ): ...
    def exchange_delete(self, exchange: str | None = None, if_unused: bool = False): ...
    def exchange_bind(self, destination, source, routing_key: str = "", arguments: Incomplete | None = None): ...
    def exchange_unbind(
        self,
        destination: Incomplete | None = None,
        source: Incomplete | None = None,
        routing_key: str = "",
        arguments: Incomplete | None = None,
    ): ...
    def queue_declare(
        self,
        queue,
        passive: bool = False,
        durable: bool = False,
        exclusive: bool = False,
        auto_delete: bool = False,
        arguments: Incomplete | None = None,
    ): ...
    def queue_delete(self, queue, if_unused: bool = False, if_empty: bool = False): ...
    def queue_purge(self, queue): ...
    def queue_bind(self, queue, exchange, routing_key: Incomplete | None = None, arguments: Incomplete | None = None): ...
    def queue_unbind(
        self,
        queue,
        exchange: Incomplete | None = None,
        routing_key: Incomplete | None = None,
        arguments: Incomplete | None = None,
    ): ...
    def tx_select(self): ...
    def tx_commit(self): ...
    def tx_rollback(self): ...



from setuptools import setup

name = "types-pika-ts"
description = "Typing stubs for pika"
long_description = '''
## Typing stubs for pika

This is a PEP 561 type stub package for the `pika` package. It
can be used by type-checking tools like
[mypy](https://github.com/python/mypy/),
[pyright](https://github.com/microsoft/pyright),
[pytype](https://github.com/google/pytype/),
PyCharm, etc. to check code that uses
`pika`. The source for this package can be found at
https://github.com/python/typeshed/tree/main/stubs/pika. All fixes for
types and metadata should be contributed there.

The `types-pika` package contains alternate, more complete type stubs, that are maintained outside of typeshed.

See https://github.com/python/typeshed/blob/main/README.md for more details.
This package was generated from typeshed commit `ed915c5cde48329e3d1e597e60c4baee27b7854a`.
'''.lstrip()

setup(name=name,
      version="1.3.0.10",
      description=description,
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/python/typeshed",
      project_urls={
          "GitHub": "https://github.com/python/typeshed",
          "Changes": "https://github.com/typeshed-internal/stub_uploader/blob/main/data/changelogs/pika.md",
          "Issue tracker": "https://github.com/python/typeshed/issues",
          "Chat": "https://gitter.im/python/typing",
      },
      install_requires=[],
      packages=['pika-stubs'],
      package_data={'pika-stubs': ['__init__.pyi', 'adapters/__init__.pyi', 'adapters/asyncio_connection.pyi', 'adapters/base_connection.pyi', 'adapters/blocking_connection.pyi', 'adapters/gevent_connection.pyi', 'adapters/select_connection.pyi', 'adapters/tornado_connection.pyi', 'adapters/twisted_connection.pyi', 'adapters/utils/__init__.pyi', 'adapters/utils/connection_workflow.pyi', 'adapters/utils/io_services_utils.pyi', 'adapters/utils/nbio_interface.pyi', 'adapters/utils/selector_ioloop_adapter.pyi', 'amqp_object.pyi', 'callback.pyi', 'channel.pyi', 'compat.pyi', 'connection.pyi', 'credentials.pyi', 'data.pyi', 'delivery_mode.pyi', 'diagnostic_utils.pyi', 'exceptions.pyi', 'exchange_type.pyi', 'frame.pyi', 'heartbeat.pyi', 'spec.pyi', 'tcp_socket_opts.pyi', 'validators.pyi', 'METADATA.toml']},
      license="Apache-2.0 license",
      classifiers=[
          "License :: OSI Approved :: Apache Software License",
          "Programming Language :: Python :: 3",
          "Typing :: Stubs Only",
      ]
)



from _typeshed import Incomplete

from pika.adapters.base_connection import BaseConnection
from pika.adapters.utils.nbio_interface import AbstractIOReference
from pika.adapters.utils.selector_ioloop_adapter import AbstractSelectorIOLoop, SelectorIOServicesAdapter

LOGGER: Incomplete

class GeventConnection(BaseConnection):
    def __init__(
        self,
        parameters: Incomplete | None = ...,
        on_open_callback: Incomplete | None = ...,
        on_open_error_callback: Incomplete | None = ...,
        on_close_callback: Incomplete | None = ...,
        custom_ioloop: Incomplete | None = ...,
        internal_connection_workflow: bool = ...,
    ) -> None: ...
    @classmethod
    def create_connection(
        cls, connection_configs, on_done, custom_ioloop: Incomplete | None = ..., workflow: Incomplete | None = ...
    ): ...

class _TSafeCallbackQueue:
    def __init__(self) -> None: ...
    @property
    def fd(self): ...
    def add_callback_threadsafe(self, callback) -> None: ...
    def run_next_callback(self) -> None: ...

class _GeventSelectorIOLoop(AbstractSelectorIOLoop):
    READ: int
    WRITE: int
    ERROR: int
    def __init__(self, gevent_hub: Incomplete | None = ...) -> None: ...
    def close(self) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    def add_callback(self, callback) -> None: ...
    def call_later(self, delay, callback): ...
    def remove_timeout(self, timeout_handle) -> None: ...
    def add_handler(self, fd, handler, events) -> None: ...
    def update_handler(self, fd, events) -> None: ...
    def remove_handler(self, fd) -> None: ...

class _GeventSelectorIOServicesAdapter(SelectorIOServicesAdapter):
    def getaddrinfo(self, host, port, on_done, family: int = ..., socktype: int = ..., proto: int = ..., flags: int = ...): ...

class _GeventIOLoopIOHandle(AbstractIOReference):
    def __init__(self, subject) -> None: ...
    def cancel(self): ...

class _GeventAddressResolver:
    def __init__(self, native_loop, host, port, family, socktype, proto, flags, on_done) -> None: ...
    def start(self) -> None: ...
    def cancel(self): ...



import builtins
from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal, Self, TypeAlias

from .amqp_object import Class, Method, Properties

# Ouch. Since str = bytes at runtime, we need a type alias for "str".
_str: TypeAlias = builtins.str  # noqa: Y042
str = builtins.bytes

PROTOCOL_VERSION: Incomplete
PORT: int
ACCESS_REFUSED: int
CHANNEL_ERROR: int
COMMAND_INVALID: int
CONNECTION_FORCED: int
CONTENT_TOO_LARGE: int
FRAME_BODY: int
FRAME_END: int
FRAME_END_SIZE: int
FRAME_ERROR: int
FRAME_HEADER: int
FRAME_HEADER_SIZE: int
FRAME_HEARTBEAT: int
FRAME_MAX_SIZE: int
FRAME_METHOD: int
FRAME_MIN_SIZE: int
INTERNAL_ERROR: int
INVALID_PATH: int
NOT_ALLOWED: int
NOT_FOUND: int
NOT_IMPLEMENTED: int
NO_CONSUMERS: int
NO_ROUTE: int
PERSISTENT_DELIVERY_MODE: int
PRECONDITION_FAILED: int
REPLY_SUCCESS: int
RESOURCE_ERROR: int
RESOURCE_LOCKED: int
SYNTAX_ERROR: int
TRANSIENT_DELIVERY_MODE: int
UNEXPECTED_FRAME: int

class Connection(Class):
    INDEX: ClassVar[int]

    class Start(Method):
        INDEX: ClassVar[int]
        version_major: Incomplete
        version_minor: Incomplete
        server_properties: Incomplete
        mechanisms: Incomplete
        locales: Incomplete
        def __init__(
            self,
            version_major: int = 0,
            version_minor: int = 9,
            server_properties: Incomplete | None = None,
            mechanisms: _str = "PLAIN",
            locales: _str = "en_US",
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class StartOk(Method):
        INDEX: ClassVar[int]
        client_properties: Incomplete
        mechanism: Incomplete
        response: Incomplete
        locale: Incomplete
        def __init__(
            self,
            client_properties: Incomplete | None = None,
            mechanism: _str = "PLAIN",
            response: Incomplete | None = None,
            locale: _str = "en_US",
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Secure(Method):
        INDEX: ClassVar[int]
        challenge: Incomplete
        def __init__(self, challenge: Incomplete | None = None) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class SecureOk(Method):
        INDEX: ClassVar[int]
        response: Incomplete
        def __init__(self, response: Incomplete | None = None) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Tune(Method):
        INDEX: ClassVar[int]
        channel_max: Incomplete
        frame_max: Incomplete
        heartbeat: Incomplete
        def __init__(self, channel_max: int = 0, frame_max: int = 0, heartbeat: int = 0) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class TuneOk(Method):
        INDEX: ClassVar[int]
        channel_max: Incomplete
        frame_max: Incomplete
        heartbeat: Incomplete
        def __init__(self, channel_max: int = 0, frame_max: int = 0, heartbeat: int = 0) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Open(Method):
        INDEX: ClassVar[int]
        virtual_host: Incomplete
        capabilities: Incomplete
        insist: Incomplete
        def __init__(self, virtual_host: _str = "/", capabilities: _str = "", insist: bool = False) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class OpenOk(Method):
        INDEX: ClassVar[int]
        known_hosts: Incomplete
        def __init__(self, known_hosts: _str = "") -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Close(Method):
        INDEX: ClassVar[int]
        reply_code: Incomplete
        reply_text: Incomplete
        class_id: Incomplete
        method_id: Incomplete
        def __init__(
            self,
            reply_code: Incomplete | None = None,
            reply_text: _str = "",
            class_id: Incomplete | None = None,
            method_id: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class CloseOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Blocked(Method):
        INDEX: ClassVar[int]
        reason: Incomplete
        def __init__(self, reason: _str = "") -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Unblocked(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class UpdateSecret(Method):
        INDEX: ClassVar[int]
        new_secret: Incomplete
        reason: Incomplete
        def __init__(self, new_secret, reason) -> None: ...
        @property
        def synchronous(self): ...
        mechanisms: Incomplete
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class UpdateSecretOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

class Channel(Class):
    INDEX: ClassVar[int]

    class Open(Method):
        INDEX: ClassVar[int]
        out_of_band: Incomplete
        def __init__(self, out_of_band: _str = "") -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class OpenOk(Method):
        INDEX: ClassVar[int]
        channel_id: Incomplete
        def __init__(self, channel_id: _str = "") -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Flow(Method):
        INDEX: ClassVar[int]
        active: Incomplete
        def __init__(self, active: Incomplete | None = None) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class FlowOk(Method):
        INDEX: ClassVar[int]
        active: Incomplete
        def __init__(self, active: Incomplete | None = None) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Close(Method):
        INDEX: ClassVar[int]
        reply_code: Incomplete
        reply_text: Incomplete
        class_id: Incomplete
        method_id: Incomplete
        def __init__(
            self,
            reply_code: Incomplete | None = None,
            reply_text: _str = "",
            class_id: Incomplete | None = None,
            method_id: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class CloseOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

class Access(Class):
    INDEX: ClassVar[int]

    class Request(Method):
        INDEX: ClassVar[int]
        realm: Incomplete
        exclusive: Incomplete
        passive: Incomplete
        active: Incomplete
        write: Incomplete
        read: Incomplete
        def __init__(
            self,
            realm: _str = "/data",
            exclusive: bool = False,
            passive: bool = True,
            active: bool = True,
            write: bool = True,
            read: bool = True,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class RequestOk(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        def __init__(self, ticket: int = 1) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

class Exchange(Class):
    INDEX: ClassVar[int]

    class Declare(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        exchange: Incomplete
        type: Incomplete
        passive: Incomplete
        durable: Incomplete
        auto_delete: Incomplete
        internal: Incomplete
        nowait: Incomplete
        arguments: Incomplete
        def __init__(
            self,
            ticket: int = 0,
            exchange: Incomplete | None = None,
            type=...,
            passive: bool = False,
            durable: bool = False,
            auto_delete: bool = False,
            internal: bool = False,
            nowait: bool = False,
            arguments: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class DeclareOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self) -> Literal[False]: ...
        def decode(self, encoded: bytes, offset: int = 0) -> Self: ...
        def encode(self) -> list[bytes]: ...

    class Delete(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        exchange: Incomplete
        if_unused: Incomplete
        nowait: Incomplete
        def __init__(
            self, ticket: int = 0, exchange: Incomplete | None = None, if_unused: bool = False, nowait: bool = False
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class DeleteOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Bind(Method):
        INDEX: ClassVar[int]
        ticket: int
        destination: Incomplete | None
        source: Incomplete | None
        routing_key: _str
        nowait: bool
        arguments: Incomplete | None
        def __init__(
            self,
            ticket: int = 0,
            destination: Incomplete | None = None,
            source: Incomplete | None = None,
            routing_key: _str = "",
            nowait: bool = False,
            arguments: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class BindOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Unbind(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        destination: Incomplete
        source: Incomplete
        routing_key: Incomplete
        nowait: Incomplete
        arguments: Incomplete
        def __init__(
            self,
            ticket: int = 0,
            destination: Incomplete | None = None,
            source: Incomplete | None = None,
            routing_key: _str = "",
            nowait: bool = False,
            arguments: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class UnbindOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

class Queue(Class):
    INDEX: ClassVar[int]

    class Declare(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        queue: Incomplete
        passive: Incomplete
        durable: Incomplete
        exclusive: Incomplete
        auto_delete: Incomplete
        nowait: Incomplete
        arguments: Incomplete
        def __init__(
            self,
            ticket: int = 0,
            queue: _str = "",
            passive: bool = False,
            durable: bool = False,
            exclusive: bool = False,
            auto_delete: bool = False,
            nowait: bool = False,
            arguments: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self) -> Literal[True]: ...
        def decode(self, encoded: bytes, offset: int = 0) -> Self: ...
        def encode(self) -> list[bytes]: ...

    class DeclareOk(Method):
        INDEX: ClassVar[int]
        queue: _str
        message_count: int
        consumer_count: int
        def __init__(self, queue: _str, message_count: int, consumer_count: int) -> None: ...
        @property
        def synchronous(self) -> Literal[False]: ...
        def decode(self, encoded: bytes, offset: int = 0) -> Self: ...
        def encode(self) -> list[bytes]: ...

    class Bind(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        queue: Incomplete
        exchange: Incomplete
        routing_key: Incomplete
        nowait: Incomplete
        arguments: Incomplete
        def __init__(
            self,
            ticket: int = 0,
            queue: _str = "",
            exchange: Incomplete | None = None,
            routing_key: _str = "",
            nowait: bool = False,
            arguments: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class BindOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Purge(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        queue: Incomplete
        nowait: Incomplete
        def __init__(self, ticket: int = 0, queue: _str = "", nowait: bool = False) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class PurgeOk(Method):
        INDEX: ClassVar[int]
        message_count: Incomplete
        def __init__(self, message_count: Incomplete | None = None) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Delete(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        queue: Incomplete
        if_unused: Incomplete
        if_empty: Incomplete
        nowait: Incomplete
        def __init__(
            self, ticket: int = 0, queue: _str = "", if_unused: bool = False, if_empty: bool = False, nowait: bool = False
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class DeleteOk(Method):
        INDEX: ClassVar[int]
        message_count: Incomplete
        def __init__(self, message_count: Incomplete | None = None) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Unbind(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        queue: Incomplete
        exchange: Incomplete
        routing_key: Incomplete
        arguments: Incomplete
        def __init__(
            self,
            ticket: int = 0,
            queue: _str = "",
            exchange: Incomplete | None = None,
            routing_key: _str = "",
            arguments: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class UnbindOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

class Basic(Class):
    INDEX: ClassVar[int]

    class Qos(Method):
        INDEX: ClassVar[int]
        prefetch_size: Incomplete
        prefetch_count: Incomplete
        global_qos: Incomplete
        def __init__(self, prefetch_size: int = 0, prefetch_count: int = 0, global_qos: bool = False) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class QosOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Consume(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        queue: Incomplete
        consumer_tag: Incomplete
        no_local: Incomplete
        no_ack: Incomplete
        exclusive: Incomplete
        nowait: Incomplete
        arguments: Incomplete
        def __init__(
            self,
            ticket: int = 0,
            queue: _str = "",
            consumer_tag: _str = "",
            no_local: bool = False,
            no_ack: bool = False,
            exclusive: bool = False,
            nowait: bool = False,
            arguments: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class ConsumeOk(Method):
        INDEX: ClassVar[int]
        consumer_tag: Incomplete
        def __init__(self, consumer_tag: Incomplete | None = None) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Cancel(Method):
        INDEX: ClassVar[int]
        consumer_tag: Incomplete
        nowait: Incomplete
        def __init__(self, consumer_tag: Incomplete | None = None, nowait: bool = False) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class CancelOk(Method):
        INDEX: ClassVar[int]
        consumer_tag: Incomplete
        def __init__(self, consumer_tag: Incomplete | None = None) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Publish(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        exchange: Incomplete
        routing_key: Incomplete
        mandatory: Incomplete
        immediate: Incomplete
        def __init__(
            self, ticket: int = 0, exchange: _str = "", routing_key: _str = "", mandatory: bool = False, immediate: bool = False
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Return(Method):
        INDEX: ClassVar[int]
        reply_code: Incomplete
        reply_text: Incomplete
        exchange: Incomplete
        routing_key: Incomplete
        def __init__(
            self,
            reply_code: Incomplete | None = None,
            reply_text: _str = "",
            exchange: Incomplete | None = None,
            routing_key: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Deliver(Method):
        INDEX: ClassVar[int]
        consumer_tag: Incomplete
        delivery_tag: Incomplete
        redelivered: Incomplete
        exchange: Incomplete
        routing_key: Incomplete
        def __init__(
            self,
            consumer_tag: Incomplete | None = None,
            delivery_tag: Incomplete | None = None,
            redelivered: bool = False,
            exchange: Incomplete | None = None,
            routing_key: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Get(Method):
        INDEX: ClassVar[int]
        ticket: Incomplete
        queue: Incomplete
        no_ack: Incomplete
        def __init__(self, ticket: int = 0, queue: _str = "", no_ack: bool = False) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class GetOk(Method):
        INDEX: ClassVar[int]
        delivery_tag: Incomplete
        redelivered: Incomplete
        exchange: Incomplete
        routing_key: Incomplete
        message_count: Incomplete
        def __init__(
            self,
            delivery_tag: Incomplete | None = None,
            redelivered: bool = False,
            exchange: Incomplete | None = None,
            routing_key: Incomplete | None = None,
            message_count: Incomplete | None = None,
        ) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class GetEmpty(Method):
        INDEX: ClassVar[int]
        cluster_id: Incomplete
        def __init__(self, cluster_id: _str = "") -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Ack(Method):
        INDEX: ClassVar[int]
        delivery_tag: Incomplete
        multiple: Incomplete
        def __init__(self, delivery_tag: int = 0, multiple: bool = False) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Reject(Method):
        INDEX: ClassVar[int]
        delivery_tag: Incomplete
        requeue: Incomplete
        def __init__(self, delivery_tag: Incomplete | None = None, requeue: bool = True) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class RecoverAsync(Method):
        INDEX: ClassVar[int]
        requeue: Incomplete
        def __init__(self, requeue: bool = False) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Recover(Method):
        INDEX: ClassVar[int]
        requeue: Incomplete
        def __init__(self, requeue: bool = False) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class RecoverOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Nack(Method):
        INDEX: ClassVar[int]
        delivery_tag: Incomplete
        multiple: Incomplete
        requeue: Incomplete
        def __init__(self, delivery_tag: int = 0, multiple: bool = False, requeue: bool = True) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

class Tx(Class):
    INDEX: ClassVar[int]

    class Select(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class SelectOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Commit(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class CommitOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class Rollback(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class RollbackOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

class Confirm(Class):
    INDEX: ClassVar[int]

    class Select(Method):
        INDEX: ClassVar[int]
        nowait: Incomplete
        def __init__(self, nowait: bool = False) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

    class SelectOk(Method):
        INDEX: ClassVar[int]
        def __init__(self) -> None: ...
        @property
        def synchronous(self): ...
        def decode(self, encoded, offset: int = 0): ...
        def encode(self): ...

class BasicProperties(Properties):
    CLASS: Incomplete
    INDEX: ClassVar[int]
    FLAG_CONTENT_TYPE: Incomplete
    FLAG_CONTENT_ENCODING: Incomplete
    FLAG_HEADERS: Incomplete
    FLAG_DELIVERY_MODE: Incomplete
    FLAG_PRIORITY: Incomplete
    FLAG_CORRELATION_ID: Incomplete
    FLAG_REPLY_TO: Incomplete
    FLAG_EXPIRATION: Incomplete
    FLAG_MESSAGE_ID: Incomplete
    FLAG_TIMESTAMP: Incomplete
    FLAG_TYPE: Incomplete
    FLAG_USER_ID: Incomplete
    FLAG_APP_ID: Incomplete
    FLAG_CLUSTER_ID: Incomplete
    content_type: Incomplete
    content_encoding: Incomplete
    headers: Incomplete
    delivery_mode: Incomplete
    priority: Incomplete
    correlation_id: Incomplete
    reply_to: Incomplete
    expiration: Incomplete
    message_id: Incomplete
    timestamp: Incomplete
    type: Incomplete
    user_id: Incomplete
    app_id: Incomplete
    cluster_id: Incomplete
    def __init__(
        self,
        content_type: Incomplete | None = None,
        content_encoding: Incomplete | None = None,
        headers: Incomplete | None = None,
        delivery_mode: Incomplete | None = None,
        priority: Incomplete | None = None,
        correlation_id: Incomplete | None = None,
        reply_to: Incomplete | None = None,
        expiration: Incomplete | None = None,
        message_id: Incomplete | None = None,
        timestamp: Incomplete | None = None,
        type: Incomplete | None = None,
        user_id: Incomplete | None = None,
        app_id: Incomplete | None = None,
        cluster_id: Incomplete | None = None,
    ) -> None: ...
    def decode(self, encoded, offset: int = 0): ...
    def encode(self): ...

methods: Incomplete
props: Incomplete

def has_content(methodNumber): ...
